"""
bitwarden secrets manager
"""

import json

import bitwarden_sdk
from waflibs import config, constants, log

logger = log.logger().get_logger()

API_URL = "https://api.bitwarden.com"
IDENTITY_URL = "https://identity.bitwarden.com"


class client:
    def __init__(
        self, access_token, org_id, api_url=API_URL, identity_url=IDENTITY_URL
    ):
        logger.debug(f"api url: {api_url}")
        logger.debug(f"identity url: {identity_url}")

        self.org_id = org_id
        self.client = bitwarden_sdk.BitwardenClient(
            bitwarden_sdk.client_settings_from_dict(
                {
                    "apiUrl": api_url,
                    "deviceType": bitwarden_sdk.DeviceType.SDK,
                    "identityUrl": identity_url,
                    "userAgent": constants.USER_AGENT,
                }
            )
        )

        self.client.access_token_login(access_token)

    def get_secret(self, secret_name):
        logger.debug(f"secret name: {secret_name}")

        secrets = self.client.secrets().list(self.org_id).data.to_dict()
        logger.debug(f"all secrets: {secrets}")
        for secret in secrets["data"]:
            if secret["key"] == secret_name:
                logger.debug(f"secret to return: {secret}")

                secret_obj = self.client.secrets().get(secret["id"])
                logger.debug(f"secret object: {secret_obj}")
                sec_value = secret_obj.data.value
                try:
                    ret_secret = config.parse_json(sec_value)
                except json.JSONDecodeError as e:
                    logger.debug(f"cannot decode as json: {e}")

                    ret_secret = sec_value

                logger.debug(f"secret to return: {ret_secret}")
                return ret_secret



#!/usr/bin/env python

import os
import pathlib
import tempfile
import unittest

from waflibs import dns


class Testdns(unittest.TestCase):
    def test_origin_full_domain_shared_two(self):
        origin = dns.convert_origin("test.foo.example.com", "example.com", shared=True)

        self.assertEqual(origin, r"test.foo.")

    def test_origin_full_domain_shared_multiple(self):
        origin = dns.convert_origin(
            "test.foo.bar.example.com", "example.com", shared=True
        )

        self.assertEqual(origin, r"test.foo.bar.")

    def test_origin_full_domain(self):
        origin = dns.convert_origin("test.example.com", "example.com")

        self.assertEqual(origin, r"test.example.com.example.com.")

    def test_origin(self):
        origin = dns.convert_origin("test", "example.com")

        self.assertEqual(origin, r"test.example.com.")

    def test_origin_two(self):
        origin = dns.convert_origin("test.foo", "example.com")

        self.assertEqual(origin, r"test.foo.example.com.")

    def test_origin_multiple(self):
        origin = dns.convert_origin("test.foo.bar.baz", "example.com")

        self.assertEqual(origin, r"test.foo.bar.baz.example.com.")

    def test_origin_shared_single(self):
        origin = dns.convert_origin("test", "example.com", shared=True)

        self.assertEqual(origin, r"test")

    def test_origin_shared_multiple(self):
        origin = dns.convert_origin("test.foo.bar.baz", "example.com", shared=True)

        self.assertEqual(origin, r"test.foo.bar.baz")

    def test_origin_shared_two(self):
        origin = dns.convert_origin("test.foo", "example.com", shared=True)

        self.assertEqual(origin, r"test.foo")



#!/usr/bin/env python

import os
import pathlib
import tempfile
import unittest

from waflibs import utils


class TestUtils(unittest.TestCase):
    def test_shell_command_capture_output_strip(self):
        stdout, stderr, _ = utils.shell_command(["echo", "hello\nthere\n\n\n\n"])

        self.assertEqual(stdout, "hello\nthere")
        self.assertEqual(stderr, "")

    def test_shell_command_capture_output(self):
        stdout, stderr, _ = utils.shell_command(
            ["echo", "hello", "there"],
        )

        self.assertEqual(stdout, "hello there")
        self.assertEqual(stderr, "")

    def test_shell_command_no_capture_output(self):
        proc = utils.shell_command(
            ["echo", "hello", "there"],
            capture_output=False,
        )

        self.assertEqual(proc.stdout, None)
        self.assertEqual(proc.stderr, None)

    def test_json_write(self):
        tf = tempfile.NamedTemporaryFile()
        filename = tf.name

        json_string = """{
  "test": "yes",
  "no": "stuff"
}"""
        json_dict = {
            "test": "yes",
            "no": "stuff",
        }

        utils.write_json_file(json_dict, filename)
        tf.seek(0)

        f = open(filename, "r")
        self.assertEqual(f.read(), json_string)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""utils for arg_parse"""

import argparse
import os
import sys


class parser:
    def __init__(
        self,
        description=None,
        prog=None,
    ):
        if not prog:
            prog = os.path.basename(sys.argv[0])

        self.ap = argparse.ArgumentParser(
            description=description,
            prog=prog,
        )

    def get_parser(
        self,
    ):
        return self.ap

    def use_config_file(self, filename, help_text=None):
        """enable argument for passing in config file"""

        if not help_text:
            help_text = f"Config file (default: %(default)s)"

        self.get_parser().add_argument(
            "-c",
            "--config",
            "--config-file",
            type=str,
            default=filename,
            help=help_text,
        )

    enable_config = use_config_file
    enable_config_file = use_config_file
    use_config = use_config_file

    def enable_verbose_logging(self, count=False):
        """enable verbose logger"""

        other_args = {}
        if count:
            action = "count"
            other_args = {
                "default": 0,
            }
        else:
            action = "store_true"

        self.get_parser().add_argument(
            "-d",
            "-v",
            "--verbose",
            "--debug",
            action=action,
            help="verbose logging",
            **other_args,
        )

    use_verbose_logger = enable_verbose_logging
    use_verbose_logging = enable_verbose_logging
    use_logger = enable_verbose_logging
    enable_logger = enable_verbose_logging
    enable_verbose_logger = enable_verbose_logging

    def enable_dry_run(self, count=False, help_text=None):
        """enable dry run mode"""

        if not help_text:
            help_text = "dry run"

        if count:
            action = "count"
        else:
            action = "store_true"

        self.get_parser().add_argument(
            "-r",
            "--dry-run",
            "--dryrun",
            action=action,
            help=help_text,
            default=0,
        )

    use_dry_run = enable_dry_run



"""
file and dir utils
"""

import os
import pathlib
import shutil

from waflibs import log

logger = log.logger().get_logger()


def __write(filename, mode, contents):
    with open(filename, mode) as f:
        f.write(contents)

def write(filename, contents, append=False):
    mode = "w"
    if append:
        mode = "a"
    try:
        __write(filename, mode, contents)
    except FileNotFoundError as e:
        logger.debug(f"parent directories do not exist... creating")

        pathlib.Path(filename).parent.mkdir(parents=True, exist_ok=True)

        __write(filename, mode, contents)


def get_absolute_path(path):
    absolute_path = pathlibs.Path(os.path.expandvars(path))
    logger.debug(f"absolute path: {absolute_path}")

    return absolute_path


get_full_path = get_absolute_path
full_path = get_absolute_path


def backup(dest, dry_run=False):
    """backup path"""

    dest_bak = pathlib.Path(f"{dest}.bak")

    logger.debug(f"removing backup file: {dest_bak}")
    remove(dest_bak, dry_run=dry_run)

    logger.debug("back up entry {}".format(dest_bak))

    logger.debug(f"backing up file/dir {dest} to {dest_bak}")
    try:
        copy(dest, dest_bak, dry_run=dry_run)
    except FileNotFoundError as e:
        logger.debug(f"ignoring error: {e}")


def backup_and_remove(dest, dry_run=False):
    """backup and remove path"""

    logger.debug(f"backing up and removing {dest}")

    backup(dest, dry_run=dry_run)

    remove(dest, dry_run=dry_run)


def remove(dest, dry_run=False):
    """remove path"""

    if dry_run:
        print(f"would remove {dest}")
        return

    if os.path.exists(dest) or os.path.islink(dest):
        if dest.is_dir() and not dest.is_symlink():
            logger.debug(f"removing dest dir: {dest}")

            shutil.rmtree(dest)
        else:
            logger.debug(f"removing dest: {dest}")

            dest.unlink(missing_ok=True)
    else:
        logger.debug(f"nothing to remove for {dest}")


def get_destination_location(dest, dotfile=True, target_dir="/tmp", custom=False):
    """get standardized destination location"""

    logger.debug(f"orig dest is: {dest}")
    if custom:
        logger.debug("using custom dir")

        dest_parts = dest.parts
        dest = pathlib.Path(f"{pathlib.Path(*dest_parts[1:])}-{dest_parts[0]}")
    logger.debug(f"real orig dest is: {dest}")

    parent_dir = os.getcwd()
    logger.debug(f"parent dir is {parent_dir}")
    final_target_dest = pathlib.Path(dest).absolute().relative_to(parent_dir)
    logger.debug(f"final target dest is {final_target_dest}")

    if dotfile:
        logger.debug("dotfile set to true... adding dot to dest")

        final_dest = pathlib.Path(target_dir, f".{final_target_dest}")
    else:
        logger.debug("dotfile not set to true... not adding dot to dest")

        final_dest = pathlib.Path(target_dir, final_target_dest)
    logger.debug(f"final dest: {final_dest}")

    return final_dest


get_dest_location = get_destination_location
get_dest_dir = get_destination_location
get_dest_file = get_destination_location


def get_parent_directory(directory, absolute=True):
    """get parent directory"""

    parent_dir = pathlib.Path(directory).parent
    if absolute:
        logger.debug(f"getting absolute path for {parent_dir}")

        parent_dir = parent_dir.absolute()
    logger.debug(f"parent dir: {parent_dir}")

    return parent_dir


get_parent_dir = get_parent_directory


def copy(src, dest, dry_run=False):
    """copy path"""

    if dry_run:
        print(f"would copy {src} to {dest}")
    else:
        if not dest.parent.exists():
            dest.mkdir(parents=True)

        if src.is_dir():
            return shutil.copytree(
                src,
                dest,
                ignore_dangling_symlinks=True,
                symlinks=True,
            )
        else:
            return shutil.copy2(
                src,
                dest,
                follow_symlinks=False,
            )



"""
general utils
"""

import json
import os
import subprocess

from waflibs import log

logger = log.logger().get_logger()


def get_env_var(var, default=None):
    env_var = os.environ.get(var, default)
    logger.debug(f"env var: {env_var}")

    return env_var


get_env = get_env_var


def shell_command(
    cmd,
    cwd=None,
    dry_run=False,
    shell=False,
    check=False,
    capture_output=True,
    pipe=False,
    input=None,
    split=True,
    env=None,
):
    """execute shell command"""

    logger.debug(f"original shell command: {cmd}")
    command = cmd
    if split and type(command) is str:
        logger.debug("command is of type string... splitting")

        command = command.split(" ")

        logger.debug(f"shell command arr: {command}")
        logger.debug(f"shell command to execute: {' '.join(command)}")
    else:
        logger.debug(f"split is false and/or command type is not a string")

    if dry_run:
        return (
            "would return stdout",
            "would return stderr",
            "would return process",
        )
    else:
        logger.debug(f"env: {env}")
        logger.debug(f"input: {input}")

        stdout = None
        stderr = None
        if pipe:
            stdout = subprocess.PIPE
            capture_output = False

        process = subprocess.run(
            command,
            text=True,
            cwd=cwd,
            shell=shell,
            capture_output=capture_output,
            check=check,
            input=input,
            stdout=stdout,
            stderr=stderr,
            env=env,
        )
        if capture_output:
            return (process.stdout.strip(), process.stderr.strip(), process)
        else:
            return process


run_command = shell_command


def write_json_file(contents, filename):
    """write json file to disk"""

    if type(filename) == str:
        f = open(filename, "w")
    else:
        f = filename
    f.write(json.dumps(contents, indent=2))
    f.close()



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

import unittest

from waflibs import domain


class TestDomain(unittest.TestCase):
    pass


class TestIDNA(TestDomain):
    def test_idna(self):
        orig = "カラ"
        result = "xn--lckwg"

        self.assertEqual(domain.idna(orig), result)


if __name__ == "__main__":
    unittest.main()



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

import pathlib
from codecs import open

from setuptools import find_packages, setup

here = pathlib.Path(__file__).parent.resolve()

with open(pathlib.Path(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="waf-libs",
    version=open(pathlib.Path(here, "source", "waflibs", "VERSION")).read().strip(),
    description="waf libs",
    long_description=long_description,
    url="https://bitbucket.org/waf/waf-libs",
    author="Felix Wong",
    author_email="felix@waf.hk",
    license="Apache",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="waf-lib waf-libs waflibs waf libs waf-libs lib waflib",
    packages=find_packages("source"),
    package_dir={"": "source"},
    install_requires=pathlib.Path(here, "requirements.txt")
    .read_text(encoding="utf-8")
    .splitlines(),
    extras_require={
        "test": ["unittest", "nose"],
    },
    test_suite="nose2.collector.collector",
    include_package_data=True,
    package_data={"": ["VERSION",]},
)



"""
git utils
"""

from . import log, utils

logger = log.logger().get_logger()


def commit_to_git(message, cwd=None):
    """shell command to commit to git"""

    cmd = ["git", "commit", "--all", "--message", message]

    stdout, stderr, _ = utils.shell_command(cmd, cwd=cwd)
    return (stdout, stderr)


def push_to_git(cwd=None, *args):
    """shell command to push to git"""

    if args:
        cmd = [
            "git",
            "push",
            args,
        ]
    else:
        cmd = [
            "git",
            "push",
        ]
    logger.debug(f"current working dir: {cwd}")

    stdout, stderr, _ = utils.shell_command(cmd, cwd=cwd)
    return (stdout, stderr)


def commit_and_push_to_git(message, cwd=None, *args):
    """shell command to command and push to git"""

    commit_stdout, commit_stderr = commit_to_git(message, cwd)
    push_stdout, push_stderr = push_to_git(cwd, *args)

    return (commit_stdout, commit_stderr, push_stdout, push_stderr)



import logging

from waflibs import (
    arg_parse,
    config,
    constants,
    database,
    dns,
    error,
    filedir,
    git,
    log,
    secrets,
    text,
    uri,
    utils,
)

program_name = constants.PROGRAM_NAME

logger = logging.getLogger(program_name)
logger.debug(f"program name: {program_name}")
logger.debug(f"logger name: {logger.name}")
logger.debug(f"logger level: {logger.level}")
